@@ -1,6 +1,11 @@
 package Reddit::Client;
 
-our $VERSION = '1.292'; # 2/4/19 not live yet
+our $VERSION = '1.30'; # 3/21/19 1.30
+# 1.30 -added flair_post and flair_user
+# 1.30 -set_post_flair and set_user_flair have been renamed select_post_flair
+# 		 and select_user_flair respectively, to distinguish them from the new 
+# 		 flair functions and make their functionality more clear.
+# 1.293 -added get_web_url to Message
 # 1.292 -added crosspost_parent to Link
 # 1.291 -print_request_on_error was accidentally left on in get_links_by_id
 # TODO: make ispost, iscomment and get_type static
@@ -8,14 +13,6 @@ our $VERSION = '1.292'; # 2/4/19 not live yet
 # 1.2817-documentation update
 # 1.281 -morecomments get_collapsed args
 # 1.281 -get_collapsed can also return morecomments
-# 1.28  -get_links_by_id
-#		-get_collapsed_comments
-#		-get_comments new version
-#		-MoreComments objects
-#		-added function get_replies to Link
-#       -can change subdomain (www to old or new, etc)
-#       -removed line number message on die. should this be an option instead?
-#		-get_permalink renamed ro get_web_url in Link and Comment
 
 $VERSION = eval $VERSION;
 
@@ -127,6 +124,8 @@ use constant API_GET_MODMAIL         => 38;
 use constant API_BAN                 => 39;
 use constant API_MORECHILDREN        => 40;
 use constant API_BY_ID               => 41;
+use constant API_FLAIR               => 42;
+use constant API_DELETEFLAIR         => 43;
 
 #===============================================================================
 # Parameters
@@ -182,6 +181,8 @@ $API[API_GET_MODMAIL   ] = ['GET',  '/api/mod/conversations'  ];
 $API[API_BAN           ] = ['POST', '/r/%s/api/friend'        ];
 $API[API_MORECHILDREN  ] = ['GET',  '/api/morechildren'       ];
 $API[API_BY_ID         ] = ['GET',  '/by_id'                  ];
+$API[API_FLAIR         ] = ['POST', '/r/%s/api/flair'         ];
+$API[API_DELETEFLAIR   ] = ['POST', '/r/%s/api/deleteflair'   ];
 
 #===============================================================================
 # Class methods
@@ -531,7 +532,7 @@ sub popular_subreddits {
 #===============================================================================
 sub get_inbox {
 	my ($self, %param) = @_;
-    	my $limit     	= $param{limit}		|| DEFAULT_LIMIT;
+    my $limit     	= $param{limit}		|| DEFAULT_LIMIT;
 	my $mode	= $param{mode}		|| MESSAGES_INBOX;	
 	my $view	= $param{view}		|| MESSAGES_INBOX;
 
@@ -540,8 +541,8 @@ sub get_inbox {
 	my $query = {};
 	$query->{mark}   = $param{mark} ? 'true' : 'false';
 	$query->{sr_detail} = $param{sr_detail} if $param{sr_detail};
-        $query->{before} = $param{before} if $param{before};
-        $query->{after}  = $param{after}  if $param{after};
+    $query->{before} = $param{before} if $param{before};
+    $query->{after}  = $param{after}  if $param{after};
 	if (exists $param{limit}) { $query->{limit} = $param{limit} || 500; }
 	else 			  { $query->{limit} = DEFAULT_LIMIT;	    }
 	
@@ -1078,17 +1079,75 @@ sub submit_text {
         text     => $text,
         sr       => $subreddit,
         kind     => SUBMIT_SELF,
-	sendreplies=>$replies,
+		sendreplies=>$replies,
     });
 
     return $result->{data}{name};
 }
-# This could go in the user section or here, but it seems like it will be
+# These could go in the user section or here, but it seems like it will be
 # more commonly used for flairing posts
+sub template {
+   	my ($self, %param) = @_;
+	my $data = {}; # POST data
+	my $url_arg;   # arguments that get interpolated into the URL
+	
+	my $result = $self->api_json_request(
+		api		=> API_FLAIR,
+		args	=> [$url_arg],
+		data	=> $data
+	);
+}
+
+# flair a post, not using an existing template, just manually providing the
+# text and CSS class 
+sub flair_post {
+   	my ($self, %param) = @_;
+	my $link_fullname = fullname($param{link_id},"t3") || die "flair_post: need 'link_id'\n";
+	my $subreddit = $param{sub} || $param{subreddit} || die "flair_post: need 'subreddit'\n";
+	my $text = $param{text} ? substr($param{text}, 0, 64) : '';
+	my $css_class = $param{css_class}; # optional
+
+	my $data = { link => $link_fullname };
+	$data->{text} = $text if $text;
+	$data->{css_class} = $css_class if $css_class;
+
+	my $result = $self->api_json_request(
+		api		=> API_FLAIR,
+		args	=> [$subreddit],
+		data	=> $data
+	);
+}
+
+# flair a user, not using an existing template, just manually providing the
+# text and CSS class 
+sub flair_user {
+   	my ($self, %param) = @_;
+	my $username = $param{username} || die "flair_user: need 'link_id'\n";
+	my $text = $param{text} ? substr($param{text}, 0, 64) : '';
+	my $css_class = $param{css_class}; #optional
+	my $subreddit = $param{sub} || $param{subreddit} || die "flair_user: need 'subreddit'\n";
+
+	my $data = { name => $username };
+	$data->{text} = $text if $text;
+	$data->{css_class} = $css_class if $css_class;
+
+	my $result = $self->api_json_request(
+		api		=> API_FLAIR,
+		args	=> [$subreddit],
+		data	=> $data
+	);
+
+}
+
+#
+sub select_post_flair {
+    my ($self, %param) = @_;
+	return $self->set_post_flair(%param);
+}
 sub set_post_flair {
-    	my ($self, %param) = @_;
+   	my ($self, %param) = @_;
 	my $sub 	= $param{subreddit} || croak "Expected 'subreddit'";
-	my $post_id 	= $param{post_id} || croak "Need 'post_id'";
+	my $post_id	= $param{post_id} || croak "Need 'post_id'";
 	my $flairid	= $param{flair_template_id} || croak "need 'flair template id'";
 	my $data	= {};
 
@@ -1104,6 +1163,10 @@ sub set_post_flair {
 
 	#return @{$result->{data}{children}};
 }
+sub select_user_flair {
+    my ($self, %param) = @_;
+	return $self->set_user_flair(%param);
+}
 sub set_user_flair {
     	my ($self, %param) = @_;
 	my $sub 	= $param{subreddit} || croak "Expected 'subreddit'";
@@ -1507,7 +1570,7 @@ sub get_origin {
 # Strip the type portion of a filname (i.e. t3_), if it exists
 sub id {
 	my $id	= shift;
-	$id 	=~ s/^[tT]\d_//;
+	$id 	=~ s/^t\d_//;
 	return $id;
 }
 # accept id or fullname, always return fullname
@@ -1566,12 +1629,14 @@ __END__
 
 =pod
 
-=pod
-
 =head1 NAME
 
 Reddit::Client - A Perl wrapper for the Reddit API.
 
+=head1 VERSION
+
+1.30 3/25/19
+
 =head1 DESCRIPTION
 
 Reddit::Client handles Oauth session management and HTTP communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>. 
@@ -1580,25 +1645,24 @@ Reddit::Client handles Oauth session management and HTTP communication with Redd
 
     use Reddit::Client;
     
-    my $client_id   = "DFhtrhBgfhhRTd";
-    my $secret      = "KrDNsbeffdbILOdgbgSvSBsbfFs";
-    
-    # Create a Reddit::Client object and authorize in one step: "script" app
+    # Create a Reddit::Client object and authorize: "script"-type app
     my $reddit = new Reddit::Client(
         user_agent  => "Test script 1.0 by /u/myusername",
-        client_id   => $client_id,
-        secret      => $secret,
+        client_id   => "client_id_string",
+        secret      => "secret_string",
         username    => "reddit_username",
         password    => "reddit_password",
     );
     
-    # Create a Reddit::Client object and authorize in one step: "web" app
+    # Create a Reddit::Client object and authorize: "web"-type app
     my $reddit = new Reddit::Client(
         user_agent    => "Test script 1.0 by /u/myusername",
-        client_id     => $client_id,
-        secret        => $secret,
+        client_id     => "client_id_string",
+        secret        => "secret_string",
         refresh_token => "refresh_token",
-    );	
+    );
+
+	# Authorization can also be done separately with get_token()
     
     # Check your inbox
     my $me = $reddit->me();
@@ -1619,7 +1683,7 @@ Reddit::Client handles Oauth session management and HTTP communication with Redd
         print $post->get_web_url();
 
         if ($post->{title} =~ /some phrase/) {
-            $post->reply("hi, I'm a bot");
+            $post->reply("hi, I'm a bot, oops I'm banned already, harsh");
         }
     }
 
@@ -1736,7 +1800,7 @@ This function is an alias for C<submit_comment>, and is equivalent to
                  [ description => $description, ] [ visibility => $visibility, ] [ subreddits => [ subreddits ], ]
                  [ icon_name => $icon_name, ] [ key_color => $hex_code, ] [ weighting_scheme => $weighting_scheme, ] )
 
-Create a multireddit. The only required argument is the name. A multi can also be created with C<edit_multi>, the only difference being that C<create_multi> will fail with a HTTP 409 error if a multi with that name already exists.
+Create a multireddit. The only required argument is the name. A multi can also be created with C<edit_multi>, the only difference being that C<create_multi> will fail with a HTTP 409 error if a multi with that name already exists. As of March 2019, trying to add a banned sub to a multi will fail with a 403 Unauthorized.
 
 Returns a hash of information about the newly created multireddit.
 
@@ -1811,6 +1875,30 @@ Note that if you are updating your sub's automod (which you can do using the pag
 
 Returns a list of Subreddit objects matching the search string C<$query>. Optionally sort them by C<sort>, which can be "relevance" or "activity".
 
+=item flair_post
+
+    flair_post ( subreddit => $subreddit, link_id => $link_id_or_fullname, 
+                 [ text => $text, ] [ css_class => $css_class, ] )
+
+Flair a post with arbitrary text and css class. 
+
+`text` and `css_class` are optional. If not provided, they will remove the existing text and/or css class. One advantage of doing this through the API (as opposed to the Reddit website) is that a css class can be applied with no text at all, not even an empty string. This allows you to have automoderator react to a thread or user in ways that are completely invisible to users.
+
+=over
+
+C<css_class> can be anything; it does not have to match an existing flair template. To select a flair template from the sub's list of flair, use L<select_post_flair|https://metacpan.org/pod/Reddit::Client#select_post_flair>.
+
+C<text> will be truncated to 64 characters if longer.
+
+=back
+
+=item flair_user
+
+    flair_user ( username => $username, text => $text, 
+               [ css_class => $css_class, ] [ subreddit => $subreddit ] )
+
+Flair a user with arbitrary text and css class. Behaves exactly as L<flair_post|https://metacpan.org/pod/Reddit::Client#flair_post> except that it is given a username instead of a link ID. To select a flair template from the sub's list of flair, use L<select_user_flair|https://metacpan.org/pod/Reddit::Client#select_user_flair>.
+
 =item get_collapsed_comments
 
     get_collapsed_comments ( link_id => $link_id, children => $children,
@@ -2052,7 +2140,7 @@ Hide a post.
 
     info ( $fullname )
 
-Returns a hash of information about C<$fullname>. C<$fullname> can be any of the 8 types of thing.
+Returns a hash of information about C<$fullname>. This will be the raw information hash from Reddit, not loaded into an object of the appropriate class (because classes don't exist for every type of thing, and because Reddit periodically updates the API, creating new fields, so it's nice to have a way to look at the raw data it's returning). C<$fullname> can be any of the 8 types of thing.
 
 =item list_subreddits
 
@@ -2126,17 +2214,25 @@ Save a post or comment.
 
 Send a private message to C<$username>. C<$subject> is limited to 100 characters.
 
+=item select_post_flair
+
+    select_post_flair ( subreddit => $subreddit, post_id => $post_id_or_fullname, flair_template_id => $flair_id )
+
+Select flair for a post from among the sub's existing flair templates. C<flair_template_id> is acquired via L<get_flair_options|https://metacpan.org/pod/Reddit::Client#get_flair_options>. To flair a post without using a template, use L<flair_post|https://metacpan.org/pod/Reddit::Client#flair_post>
+
+=item select_user_flair
+
+    select_user_flair ( subreddit => $subreddit, username => $username, flair_template_id => $flair_id )
+	
+Select flair for a user from among the sub's existing flair templates. C<flair_template_id> is acquired via L<get_flair_options|https://metacpan.org/pod/Reddit::Client#get_flair_options>. To flair a user without using a template, use L<flair_user|https://metacpan.org/pod/Reddit::Client#flair_user>.
+
 =item set_post_flair
 
-    set_post_flair ( subreddit => $subreddit, post_id => $post_id_or_fullname, flair_template_id => $flair_id )
-	
-Set the flair on a post. C<flair_template_id> is acquired via C<get_flair_options()>.
+    Deprecated. Use L<select_post_flair|https://metacpan.org/pod/Reddit::Client#select_post_flair> or L<flair_post|https://metacpan.org/pod/Reddit::Client#flair_post>.
 
 =item set_user_flair
 
-    set_user_flair ( subreddit => $subreddit, username => $username, flair_template_id => $flair_id )
-	
-Set the flair for a user. C<flair_template_id> is acquired via C<get_flair_options()>.
+    Deprecated. Use L<select_user_flair|https://metacpan.org/pod/Reddit::Client#select_user_flair> or L<flair_user|https://metacpan.org/pod/Reddit::Client#flair_user>.
 
 =item submit_comment 
 
