@@ -12,7 +12,7 @@ use fields qw/link_flair_text media url link_flair_css_class num_reports
               author media_embed author_flair_css_class selftext domain
               num_comments clicked saved thumbnail subreddit_id approved_by
               selftext_html created hidden over_18 permalink morecomments
-		user_reports mod_reports/;
+		user_reports mod_reports crosspost_parent/;
 
 use constant type => "t3";
 
